unit _3DG;

interface

uses Windows, _3DR;

type
  TG3dHandle = pointer;

const
  ONE = 65536.0;
  ZONE = 268435455.0;
  COLONE = 65535.0;
  MAX_C = 255;
  SMALL_NUMBER = 1.0E-12;

  G3D_OFF = 0;
  G3D_ON = 1;

(*
// Dictionary of Acronyms, Keywords, etc.
//
// G3d 	- 3DG reordered for compilation reasons.
//
// 23F		- indicates a 2 by 3 floating point matrix
// A		- indicates alpha blending variable present as in
//			  ColorFA
// BMP		- bitmap image file format
// Coords	- coordinate system
// Eqns		- equations
// F		- indicates floating point 32 bit version
// FA		- indicates floating point 32 bit with alpha (for colors)
// FW		- indicates floating point 32 bit homogeneous coordinates
// GC		- Geometry Context
// Isect	- intersection such as the intersection of a line and
//			  a plane.
// L		- indicates Fixed point 32bit version
// LW		- indicates Fixed point 32bit homogeneous coordinates
// luBackSub- lower, upper back substitution
// luDecomp	- lower triangular, upper triangular decomposition
//			  of a matrix.
// Matt		- material properties of a surface
// Mult		- multiply
// Norm,nrm	- normal vector perpendicular to a surface.
// Pow		- power or exponentiation
// prim		- drawing primitive such as a line or polygon
// prop		- property or attribute such as of a light source
// pt		- Point in some coordinate system
// Q		- quadratic.  2nd degree (3rd order) polynomial
// qd		- quasi definitive, or quadrature, or quick and dirty.
//			  Indicates that a function is a quick approximation.
// Sqrt		- square root
// S		- indicates short (16bit) components such as for colors
// SA		- indicates short (16bit) components with alpha
// trans	- transform, a 4x4 matrix for transforming points
// vec		- vector, usually x,y,z
// vtx		- vertex such as a polygon vertex
// Win		- window
// W		- indicates the homogeneous coordinate, w value is present
//			  as in PointFW_t
*)


(* ----------------------CONTEXT--------------------------- *)

type
  TG3dContextProps = (G3D_NO_DRAW_ORDER);

  TG3dState = (G3DL_FACETED_NORMALS,
		G3DL_FLAT_SHADING,
		G3DL_TEXTURE_MOD,
		G3DL_FRONT_CCW,
		G3DL_NO_CLIPPING);

procedure G3dSetState(hGC : TG3dHandle; st : TG3dState; data : pointer);
procedure G3dGetState(hGC : TG3dHandle; st : TG3dState; data : pointer);

type
  TG3dCoords = (G3DC_OBJECT_COORDS,
		G3DC_WINDOW_COORDS);

  TG3dCapabilities = record
    zbufferPlanes : TFixed32;
  end;

function G3dCreateGC(hRC : TR3dHandle) : TG3dHandle;

procedure G3dDeleteGC(hGC : TG3dHandle);

procedure G3dSetCoords(hGC : TG3dHandle; coords : TG3dCoords);

procedure G3dQuerySystemCapabilities(hGC : TG3dHandle; var cap : TG3dCapabilities);

(* ----------------------COLORS------------------------ *)

type
  TColorF = record
    r,g,b : single;
  end;

  TColorFA = record
    r,g,b,a : single;
  end;

  TColorSA = record
    r,g,b,a : TFixed16;
  end;

(* ----------------------POINTS------------------------ *)

  TPointL = record
    x,y,z : TFixed32;
  end;

  TPointLW = record
    x,y,z,w : TFixed32;
  end;

  TPointF = record
    x,y,z : single;
  end;

  TPointFW = record
    x,y,z,w : single;
  end;

  TLineF = record
    point, vector : TPointF;
  end;

  TPlaneF = record
    a,b,c,d : single;
  end;

  TSphereF = record
    center : TPointF;
    r : single;
  end;

  TLineL = record
    point, vector : TPointL;
  end;

  TPlaneL = record
    a,b,c,d : TFixed32;
  end;

  TSphereL = record
    center : TPointL;
    r : TFixed32;
  end;

(* ----------------------PRIMITIVES------------------------ *)

const
  G3D_MAX_VERTICES	= 256;
  G3D_MAX_LIST		= 4096;
  G3D_MAX_PLANES	= TFixed32(8);

type
  TG3dCPlaneProp = (G3DCP_DATA, G3DCP_SIGN);

  TG3dPrimitive = longint;

procedure G3dGetClipPlane(hGC : TG3dHandle; wCPPNum : TFixed32;
  prop : TG3dCPlaneProp; data : pointer); stdcall;

procedure G3dSetClipPlane(hGC : TG3dHandle; wCPPNum : TFixed32;
  prop : TG3dCPlaneProp; data : pointer); stdcall;

const
  G3D_PRM_POINTS = R3D_PRM_POINTS;
  G3D_PRM_LINES = R3D_PRM_LINES;
  G3D_PRM_POLYLINE = R3D_PRM_POLYLINE;
  G3D_PRM_TRIANGLES = R3D_PRM_TRIANGLES;
  G3D_PRM_QUADS = R3D_PKT_QUADS;
  G3D_PRM_TRISTRIP = R3D_PRM_TRISTRIP;
  G3D_PRM_QUADSTRIP = R3D_PKT_QUADSTRIP;
  G3D_PRM_POLYGON = R3D_PRM_POLYGON;

procedure G3dQPatchF(hGC : TG3dHandle; un : TFixed32; vn : TFixed32;
  p : pointer; pstep : TFixed32; t : pointer; tstep : TFixed32;
  c : pointer; cstep : TFixed32; NormFlag : longint); stdcall;

procedure G3dQPatchListF(hGC : TG3dHandle; un : TFixed32; vn : TFixed32;
  qp : pointer; qpstep : TFixed32; qt : pointer; qtstep : TFixed32;
  qc : pointer; qcstep : TFixed32; NormFlag : longint;
  p : pointer; pstep : TFixed32; n : pointer; nstep : TFixed32; t : pointer; tstep : TFixed32;
  c : pointer; cstep : TFixed32; idx : pointer); stdcall;

procedure G3dQPatchL(hGC : TG3dHandle; un : TFixed32; vn : TFixed32;
  p : pointer; pstep : TFixed32; t : pointer; tstep : TFixed32;
  c : pointer; cstep : TFixed32; NormFlag : longint); stdcall;

procedure G3dQPatchListL(hGC : TG3dHandle; un : TFixed32; vn : TFixed32;
  qp : pointer; qpstep : TFixed32; qt : pointer; qtstep : TFixed32;
  qc : pointer; qcstep : TFixed32; NormFlag : longint;
  p : pointer; pstep : TFixed32; n : pointer; nstep : TFixed32; t : pointer; tstep : TFixed32;
  c : pointer; cstep : TFixed32; idx : pointer); stdcall;


procedure G3dPrimitiveF(hGC : TG3dHandle; prim : TG3dPrimitive; nvtx : TFixed32;
  p : pointer; pstep : TFixed32; n : pointer; nstep : TFixed32;
  t : pointer; tstep : TFixed32; c : pointer; cstep : TFixed32); stdcall;

procedure G3dPrimitiveL(hGC : TG3dHandle; prim : TG3dPrimitive; nvtx : TFixed32;
  p : pointer; pstep : TFixed32; n : pointer; nstep : TFixed32;
  t : pointer; tstep : TFixed32; c : pointer; cstep : TFixed32); stdcall;

procedure G3dBeginPrim(hGC : TG3dHandle; prim : TG3dPrimitive; nvtx : TFixed32); stdcall;

procedure G3dAddPrimVtxF(hGC : TG3dHandle; var p,n,t : TPointF; var c : TColorFA); stdcall;

procedure G3dAddPrimVtxL(hGC : TG3dHandle; var p,n,t : TPointL; var c : TColorSA); stdcall;

procedure G3dEndPrim(hGC : TG3dHandle); stdcall;

procedure G3dPolygonListF(hGC : TG3dHandle; npoly, tvtx : TFixed32;
  p : pointer; pstep : TFixed32; n : pointer; nstep : TFixed32;
  t : pointer; tstep : TFixed32; c : pointer; cstep : TFixed32;
  polylist : pointer); stdcall;

procedure G3dPolygonListL(hGC : TG3dHandle; npoly, tvtx : TFixed32;
  p : pointer; pstep : TFixed32; n : pointer; nstep : TFixed32;
  t : pointer; tstep : TFixed32; c : pointer; cstep : TFixed32;
  polylist : pointer); stdcall;

procedure G3dPolygonListFF(hGC : TG3dHandle; npoly, tvtx : TFixed32;
  p : pointer; pstep : TFixed32; n : pointer; nstep : TFixed32;
  fn : pointer; fnstep : TFixed32;
  t : pointer; tstep : TFixed32; c : pointer; cstep : TFixed32;
  polylist : pointer); stdcall;

procedure G3dPolygonListFL(hGC : TG3dHandle; npoly, tvtx : TFixed32;
  p : pointer; pstep : TFixed32; n : pointer; nstep : TFixed32;
  fn : pointer; fnstep : TFixed32;
  t : pointer; tstep : TFixed32; c : pointer; cstep : TFixed32;
  polylist : pointer); stdcall;

(* ----------------------LIGHTS------------------------ *)

const
  G3D_MAX_LIGHTS		= TFixed32(16);

type
  TG3dLightProp = (
		G3DL_DIMMER,	(* Float_t	  0.0	(off) *)
		G3DL_AMBIENT,	(* ColorF_t	  0.0, 0.0, 0.0	 *)
		G3DL_DIFFUSE,	(* ColorF_t	  1.0, 1.0, 1.0	 *)
		G3DL_SPECULAR,	(* ColorF_t	  1.0, 1.0, 1.0	 *)
		G3DL_LOCATION,	(* PointFW_t  0.0, 0.0, 1.0, 0.0	 *)
		G3DL_DIRECTION,	(* PointF_t 	  0.0, 0.0, -1.0	 *)
		G3DL_SPOT_EXP,	(* Float_t	  0.0		 *)
		G3DL_SPOT_ANGLE,	(* Float_t	  PI radians	 *)
		G3DL_ATTENA,	(* Float_t	  1.0		 *)
		G3DL_ATTENB,	(* Float_t	  0.0		 *)
		G3DL_ATTENC	(* Float_t	  0.0		 *)
  );

  TG3dHazeMode = (
		G3DL_OFF,
		G3DL_LINEAR,
		G3DL_EXP,
		G3DL_EXP2
  );

procedure G3dResetLight(hGC : TG3dHandle; wLightNum : TFixed32); stdcall;
procedure G3dGetLight(hGC : TG3dHandle; wLightNum : TFixed32;
  prop: TG3dLightProp; data : pointer); stdcall;
procedure G3dSetLight(hGC : TG3dHandle; wLightNum : TFixed32;
  prop: TG3dLightProp; data : pointer); stdcall;
procedure G3dSetHazeColor(hGC : TG3dHandle; var color : TColorF); stdcall;
procedure G3dSetHazeFunction(hGC : TG3dHandle; mode : TG3dHazeMode;
  density, hnear, hfar : single); stdcall;

(* ----------------------MATERIAL------------------------ *)

const
  G3D_MAX_MATTS		= 16;

type
  TG3dMattProp = (
		G3DM_AMBIENT,	(* ColorF_t		0.0, 0.0, 0.0 *)
		G3DM_DIFFUSE,	(* ColorFA_t	0.0, 0.0, 0.0, 1.0 *)
		G3DM_SPECULAR,	(* ColorF_t		0.0, 0.0, 0.0 *)
		G3DM_EMIT,	(* ColorF_t			0.0, 0.0, 0.0 *)
		G3DM_SHINE,	(* Float_t      	0.0  *)
		G3DM_CULL,	(* Float_t     		0.0  *)
		G3DM_BACK_AMBIENT,	(* ColorF_t		0.0, 0.0, 0.0 *)
		G3DM_BACK_DIFFUSE,	(* ColorFA_t	0.0, 0.0, 0.0,
					 * 1.0 *)
		G3DM_BACK_SPECULAR,	(* ColorF_t		0.0, 0.0, 0.0 *)
		G3DM_BACK_EMIT,	(* ColorF_t			0.0, 0.0, 0.0 *)
		G3DM_BACK_SHINE,	(* Float_t		0.0  *)
		G3DM_BACK_CULL,	(* Float_t  	    	0.0  *)
		G3DM_ALPHA,	(* Float_t  *)
		G3DM_TEXTURE_HDL	(* TEX_HDL  *)
  );

procedure G3dGetMatt(hGC : TG3dHandle; wMattNum : TFixed32;
  prop : TG3dMattProp; data : pointer); stdcall;
procedure G3dSetMatt(hGC : TG3dHandle; wMattNum : TFixed32;
  prop : TG3dMattProp; data : pointer); stdcall;
procedure G3dSetCurrentMatt(hGC : TG3dHandle; wMattNum : TFixed32); stdcall;

(* ----------------------TRANSFORMS------------------------ *)

const
  G3D_TRANS_STACK_SIZE		= 32;

type
  TG3dTransformStack = (
		G3DT_MODEL,
		G3DT_CAM_CLIP,
		G3DT_TEXTURE
  );

  TTransformF = array[0..3,0..3] of single;
  PTransformF = ^TTransformF;
  TTransform23F = array[0..1,0..2] of single;
  {
	typedef Float_t TransformF_t[4][4];
	typedef Float_t Transform23F_t[2][3];
  }

procedure G3dSetActiveStack(hGC : TG3dHandle; stack : TG3dTransformStack); stdcall;
procedure G3dClearStack(hGC : TG3dHandle); stdcall;
procedure G3dPushTransform(hGC : TG3dHandle); stdcall;
procedure G3dPopTransform(hGC : TG3dHandle); stdcall;
procedure G3dIdentityTransform(hGC : TG3dHandle); stdcall;
procedure G3dRotateTransform(hGC : TG3dHandle; theta, x, y, z : single); stdcall;
procedure G3dTranslateTransform(hGC : TG3dHandle; tx, ty, tz : single); stdcall;
procedure G3dScaleTransform(hGC : TG3dHandle; sx, sy, sz : single); stdcall;
procedure G3dPreMultTransform(hGC : TG3dHandle; var mat : TTransformF); stdcall;
procedure G3dPostMultTransform(hGC : TG3dHandle; var mat : TTransformF); stdcall;
procedure G3dSetTransform(hGC : TG3dHandle; var mat : TTransformF); stdcall;
procedure G3dGetTransform(hGC : TG3dHandle; var mat : TTransformF); stdcall;
procedure G3dTransposeMatrix(var mat : TTransformF); stdcall;
procedure G3dInvertMatrix(const src : TTransformF; var dest : TTransformF); stdcall;
procedure G3dMultMatrix(var m0, m1, m2 : TTransformF); stdcall;
procedure G3dCopyMatrix(var m0, m1 : TTransformF); stdcall;
procedure G3dRotateMatrix(theta, a, b, c : single; var mat : TTransformF); stdcall;
procedure G3dIdentityMatrix(var mat : TTransformF); stdcall;
function G3dUnitVector(var vec : TPointF) : single; stdcall;
procedure G3dUnitVector2(const src : TPointF; var dest : TPointF); stdcall;
function G3dluDecomp(amat : pointer; N : TFixed32;
  index : pointer; ric : single) : TFixed32; stdcall;
procedure G3dluBackSub(amat : pointer; N : TFixed32;
  index : pointer; b: single); stdcall;
procedure G3dSolveLinearEqns(amat : pointer; b : pointer; N : TFixed32); stdcall;
procedure G3dTransformPointF(const ptin : TPointF;
  var ptout : TPointFW; var mat : TTransformF); stdcall;
procedure G3dTransformNormalF(const pin : TPointF; var pout : TPointF;
  var mat : TTransformF); stdcall;
function G3dGetModelCamMatrix(hGC : TG3dHandle) : PTransformF; stdcall;
function G3dGetInverseModelCamMatrix(hGC : TG3dHandle) : PTransformF; stdcall;
procedure G3dTranslateMatrix(x, y, z : single; var mat : TTransformF); stdcall;
procedure G3dScaleMatrix(x, y, z : single; var mat : TTransformF); stdcall;

(* ----------------------CAMERAS-------------------------- *)

type
  TG3dProjection = (
		G3DP_OFF,
		G3DP_PARALLEL,
		G3DP_PERSPECTIVE
  );

procedure G3dGetCameraProjection(hGC : TG3dHandle; proj : TG3dProjection); stdcall;
procedure G3dGetCameraPosition(hGC : TG3dHandle; var loc : TPointFW;
  var dir, up : TPointF); stdcall;
procedure G3dGetCameraView(hGC : TG3dHandle;
  var right, left, top, bottom, pnear, pfar : single); stdcall;
procedure G3dGetCameraPort(hGC : TG3dHandle;
  var right, left, top, bottom : single); stdcall;
procedure G3dSetCameraProjection(hGC : TG3dHandle; proj : TG3dProjection); stdcall;
procedure G3dSetCameraPosition(hGC : TG3dHandle; var loc : TPointFW;
  var dir, up : TPointF); stdcall;
procedure G3dSetCameraView(hGC : TG3dHandle;
  right, left, top, bottom, pnear, pfar : single); stdcall;
procedure G3dSetCameraPort(hGC : TG3dHandle;
  right, left, top, bottom : single); stdcall;
procedure G3dSetWindowSize(hGC : TG3dHandle; ww, wh : single); stdcall;

(* ----------SELECTION & MANIPULATION----------------------- *)

type
  TG3dTrackballEvent = (
		TB_START,
		TB_MOVE,
		TB_END
  );

  {
	Void_t  G3dTrackball(G3dTrackballEvent_t event, PointF_t * center, Float_t radius,
		    Float_t x, Float_t y, PointF_t *axis, Float_t *theta);
	Void_t  G3dPointWindowToObject(G3dHandle_t hGC, Float_t x, Float_t y, LineF_t * line);
	Void_t  G3dLineWindowToObject(G3dHandle_t hGC, Float_t x0, Float_t y0, Float_t x1, Float_t y1,
		    PlaneF_t * plane);
	Void_t  G3dVectorWindowToObject(G3dHandle_t hGC, Float_t x0, Float_t y0, Float_t x1, Float_t y1,
		    LineF_t * vec);
	Void_t  G3d3PointPlane(PointF_t * a, PointF_t * b, PointF_t * c, PlaneF_t * plane);
	Void_t  G3dPointNormalPlane(PointF_t * p, PointF_t * n, PlaneF_t * plane);
	Fixed32_t  G3dLinePlaneIntersect(LineF_t * line, PlaneF_t * plane, PointF_t * vtx);
	Fixed32_t  G3dLineSphereIntersect(LineF_t * line, SphereF_t * sphere, PointF_t * vtx);
	Void_t  G3dBoundingSphere(Fixed32_t npts, PointF_t * pts, SphereF_t * sphere);
  }

(* ---------------------- UTILITIES ------------------------------- *)

function G3dqdsqrt(val : single) : single; stdcall;     (* quick and dirty square root *)
function G3dqdpow(x : single; exp : TFixed32) : single; stdcall;(* quick and dirty power *)
function G3dqdsin(val : single) : single; stdcall;      (* quick and dirty sin *)
function G3dqdcos(val : single) : single; stdcall;      (* quick and dirty cos *)

function G3dVecLengthFloat(x,y,z : single) : single;
function G3dVecLengthFixed(x,y,z : TFixed32) : single;

(*
#define G3dVecLength(x,y,z) ((Float_t)sqrt((x)*(x)+(y)*(y)+(z)*(z)))

#define G3dVecLengthPt(p) ((Float_t)sqrt(((p)->x)*((p)->x)+((p)->y)*((p)->y)+((p)->z)*((p)->z)))

#define G3dCross( A, B, C ) { \
  (C)->x =  (A)->y * (B)->z - (A)->z * (B)->y; \
  (C)->y = -(A)->x * (B)->z + (A)->z * (B)->x; \
  (C)->z =  (A)->x * (B)->y - (A)->y * (B)->x; \
   }

#define G3dDot(a,b) ((a)->x*(b)->x+(a)->y*(b)->y+(a)->z*(b)->z)

(* ---------------------- IMAGES ------------------------------- *)

function G3dReadBMPFile(ptrToFilename : PChar; dibHandle : THandle) : THandle; stdcall;
function G3dLoadTexture(hGC : TG3dHandle; ptrToFilename : PChar) : TR3dHandle; stdcall;
function G3dCreateTexture(hGC : TG3dHandle; dibHandle : THandle) : TR3dHandle; stdcall;
procedure G3dTransformBMP(bmp1 : THandle; var bmp2 : THandle;
  var xform : TTransform23F; x0, y0, w, h, bpp, rgb : integer); stdcall;

implementation

function G3dVecLengthFloat(x,y,z : single) : single;
begin
  Result := sqrt(sqr(x)+sqr(y)+sqr(z));
end;

function G3dVecLengthFixed(x,y,z : TFixed32) : single;
begin
  Result := sqrt(sqr(x)+sqr(y)+sqr(z));
end;

end.
